@@ -131,11 +131,13 @@
 	      ";\n"
 	      "struct foo x;\n"
 	      ))
+       (code "int len = sizeof(\"abc\" \"def\");\n")
        (tree (parse-string code))
+       (expr (sx-ref* tree 1 2 1 2 1))
        ;;(tree (parse-file "zz.c"))
        ;;(udict (c99-trans-unit->udict/deep tree))
-       (decl1 (sx-ref tree 1))
-       (udict (unitize-decl decl1 '()))
+       ;;(decl1 (sx-ref tree 1))
+       ;;(udict (unitize-decl decl1 '()))
        ;;(udecl (udict-struct-ref udict "epoll_event"))
        ;;(udecl (stripdown-udecl udecl))
        ;;(udecl (expand-typerefs udecl udict))
@@ -144,7 +146,9 @@
        ;;(xdecl (expand-typerefs-in-code code 2))
        )
   (pp tree)
-  (pp udict)
+  (pp expr)
+  (pp (eval-c99-cx expr))
+  ;;(pp udict)
   ;;(pp udecl)
   ;;(pp mdecl)
   ;;(pp xdecl)
