@@ -26,10 +26,7 @@
   #:use-module (nyacc lang util)
   #:use-module (rnrs arithmetic bitwise)
   #:use-module ((srfi srfi-43) #:select (vector-map vector-for-each))
-  #:use-module (system foreign)
-  )
-(use-modules (ice-9 pretty-print))
-(define pp pretty-print)
+  #:use-module (system foreign))
 
 (define c99cx-spec
   (lalr-spec
