@@ -28,7 +28,9 @@
 ;; bytestructure->descriptor->ffi-descriptor
 ;; bs:pointer->proc
 
-;; TODO: add renamer
+;; TODOs
+;; 1) add renamer
+;; 2) think about cnvt-fctn that generates C code
 
 ;; Issue:
 ;; So issue is when 'typedef struct ref foo_t' has no 'struct def'
@@ -58,7 +60,6 @@
 	    ;;pkg-config-incs pkg-config-defs pkg-config-libs
 	    ;; debugging
 	    ;;ffi-symmap
-	    fhoo
 	    )
   #:use-module (nyacc lang c99 cpp)
   #:use-module (nyacc lang c99 parser)
@@ -80,15 +81,14 @@
   #:use-module (srfi srfi-11)
   #:use-module (srfi srfi-37)
   #:use-module (system base pmatch)
+  #:use-module ((system base compile) #:select (compile-file))
   #:use-module (ice-9 format)
   #:use-module (ice-9 popen)
   #:use-module (ice-9 rdelim)
   #:use-module (ice-9 regex)
+  #:use-module (ice-9 pretty-print)
   #:re-export (*nyacc-version*)
-  #:version (0 86 9))
-
-(define (fhoo) #t)			; debugging
-(use-modules (ice-9 pretty-print))
+  #:version (0 90 0))
 
 (define fh-cpp-defs
   (cond
@@ -149,8 +149,6 @@
 (define* (upscm tree #:key (per-line-prefix ""))
   (ugly-print tree (*mport*) #:per-line-prefix per-line-prefix))
 (define (c99scm tree)
-  #;(when (equal? "epoll_event" (sx-ref* tree 1 1 1 1 1))
-    (sferr "c99scm:\n") (pperr tree))
   (pretty-print-c99 tree
 		    (*mport*)
 		    #:per-line-prefix ";; "))
@@ -253,6 +251,8 @@
    '()
    (pkg-config name "--cflags")))
 
+;; TODO:
+;; 1) check if --libs provides -L that is not in ld.so dirs.
 (define (pkg-config-libs name)
   (fold-right
    (lambda (s l)
@@ -387,27 +387,6 @@
 	cx))
     (lambda (key . args) (sferr "const-expr->number failed: ~S\n" expr) #f)))
 
-#!
-;; given a union-descriptor geneate a bounding struct-descriptor
-(define (bounding-struct-descriptor union-descriptor)
-  (let ((size (bytestructure-descriptor-size union-descriptor))
-	(align (bytestructure-descriptor-alignment union-descriptor))
-	(diff (- size align)))
-    (if (positive? diff)
-	(case align
-	  ((8) (bs:struct `(x ,double) `(y ,(bs-vector diff uint8))))
-	  ((4) (bs:struct `(x ,uint32) `(y ,(bs-vector diff uint8))))
-	  ((2) (bs:struct `(x ,uint16) `(y ,(bs-vector diff uint8))))
-	  ((1) (bs:struct `(x ,uint8) `(y ,(bs-vector diff uint8))))
-	  (else (error "unknown alignment")))
-	(case align
-	  ((8) (bs:struct `(x ,double)))
-	  ((4) (bs:struct `(x ,uint32)))
-	  ((2) (bs:struct `(x ,uint16)))
-	  ((1) (bs:struct `(x ,uint8)))
-	  (else (error "unknown alignment"))))))
-!#
-
 ;; just the type, so parent has to build the name-value pairs for
 ;; struct members
 (define (mtail->bs-desc mdecl-tail)
@@ -705,8 +684,7 @@
   (sfscm "   ((integer? n) n)\n")
   (sfscm "   (else (error \"bad arg\"))))\n")
   (sfscm "(define-public (wrap-~A v)\n" name)
-  (sfscm "  (assq-ref ~A-enum-vnl v))\n" name)
-  )
+  (sfscm "  (assq-ref ~A-enum-vnl v))\n" name))
 
 (define (cnvt-enum-def typename enum-name enum-def-list)
   (let* ((name-val-l
@@ -759,8 +737,7 @@
     ("signed long long" . ffi:long) ("signed long long int" . ffi:long)
     ("unsigned long long" . ffi:unsigned-long)
     ("unsigned long long int" . ffi:unsigned-long)
-    ("_Bool" . ffi:int8)
-    ))
+    ("_Bool" . ffi:int8)))
 
 (define ffi-defined (map car ffi-typemap))
 
@@ -772,8 +749,7 @@
     (ffi:ssize_t . ,ssize_t) (ffi:ptrdiff_t . ,ptrdiff_t) (ffi:int8 . ,int8)
     (ffi:uint8 . ,uint8) (ffi:int16 . ,int16) (ffi:uint16 . ,uint16)
     (ffi:int32 . ,int32) (ffi:uint32 . ,uint32) (ffi:int64 . ,int64)
-    (ffi:uint64 . ,uint64) (ffi-void* . *)
-    ))
+    (ffi:uint64 . ,uint64) (ffi-void* . *)))
 
 (define (mtail->ffi-desc mdecl-tail)
   ;;(sferr "mdecl=~S\n" mdecl)
@@ -1217,6 +1193,7 @@
 		((specl declr) (cleanup-udecl specl declr))
 		((clean-udecl) (values (sx-list tag #f specl declr)))
 		)
+    ;;(sferr "clean udecl:\n") (pperr clean-udecl)
     (sxml-match clean-udecl
 
       ;; typedef void **ptr_t;
@@ -1395,25 +1372,29 @@
 	;; This case represents three possible uses:
 	((member struct-name defined)
 	 ;; 1) struct defined previously
+	 ;;(sferr " CASE1\n")
 	 (sfscm "(define-public ~A-desc struct-~A-desc)\n" typename struct-name)
 	 (sfscm "(define-public ~A*-desc struct-~A*-desc)\n"
 		typename struct-name))
 	((udict-struct-ref udict struct-name) =>
 	 ;; 2) struct defined later
 	 (lambda (struct-decl)
+	   ;;(sferr " CASE2 (~S)\n" struct-name)
 	   (back-ref-extend! struct-decl typename)
 	   (sfscm "(define-public ~A-desc 'void)\n" typename)
+	   (sfscm "(define-public ~A fh-void)\n" typename)
+	   (sfscm "(define-public ~A? fh-void?)\n" typename)
+	   (sfscm "(define-public make-~A make-fh-void)\n" typename)
 	   (sfscm "(define-public ~A*-desc (bs:pointer (delay ~A-desc)))\n"
-		  typename typename)))
+		  typename typename)
+	   ))
 	(else
 	 ;; 3) struct never defined; only used as pointer
+	 ;;(sferr " CASE3\n")
 	 (sfscm "(define-public ~A-desc 'void)\n" typename)
-	 ;; ----- This may be a hack? Write it up! ------------
-	 ;; Added to get glugl GLUnurbsObj to work.
 	 (sfscm "(define-fh-type-alias ~A fh-void)\n" typename)
 	 (sfscm "(define-public ~A? fh-void?)\n" typename)
 	 (sfscm "(define-public make-~A make-fh-void)\n" typename)
-	 ;; --------------------------------------------------
 	 (sfscm "(define-public ~A*-desc (bs:pointer ~A-desc))\n"
 		typename typename)))
        (fhscm-def-pointer (sw/* typename))
@@ -1484,6 +1465,9 @@
 	 (lambda (union-decl)
 	   (back-ref-extend! union-decl typename)
 	   (sfscm "(define-public ~A-desc 'void)\n" typename)
+	   (sfscm "(define-public ~A fh-void)\n" typename)
+	   (sfscm "(define-public ~A? fh-void?)\n" typename)
+	   (sfscm "(define-public make-~A make-fh-void)\n" typename)
 	   (sfscm "(define-public ~A*-desc (bs:pointer (delay ~A-desc)))\n"
 		  typename typename)))
 	(else
@@ -1590,7 +1574,7 @@
 	   (sfscm "(define-public make-~A* make-~A*)\n" typename name))
 	 (values (cons typename wrapped) (cons typename defined)))
 	(else
-	 (let ((xdecl (expand-typerefs udecl udict defined)))
+	 (let ((xdecl (expand-typerefs udecl (*udict*) defined)))
 	   (cnvt-udecl xdecl udict wrapped defined)))))
 
       ;; === structs and unions ==========
@@ -1604,9 +1588,11 @@
 	 (lambda (name-list)
 	   (cnvt-struct-def attr1 #f struct-name field-list)
 	   (for-each
-	    (lambda (name)
-	      (sfscm "(set! ~A-desc struct-~A-desc)\n" name struct-name)
-	      (fhscm-def-compound name))
+	    (lambda (typename)
+	      (sfscm "(set! ~A-desc struct-~A-desc)\n" typename struct-name)
+	      (fhscm-def-compound typename)
+	      (fhscm-def-pointer (sw/* typename))
+	      (fhscm-ref-deref typename))
 	    name-list)
 	   (values (cons (w/struct struct-name) wrapped)
 		   (cons (w/struct struct-name) defined))))
@@ -1635,9 +1621,11 @@
 	 (lambda (name-list)
 	   (cnvt-union-def #f #f union-name field-list)
 	   (for-each
-	    (lambda (name)
-	      (sfscm "(set! ~A-desc union-~A-desc)\n" name union-name)
-	      (fhscm-def-compound name))
+	    (lambda (typename)
+	      (sfscm "(set! ~A-desc union-~A-desc)\n" typename union-name)
+	      (fhscm-def-compound typename)
+	      (fhscm-def-pointer (sw/* typename))
+	      (fhscm-ref-deref typename))
 	    name-list)
 	   (values (cons (w/union union-name) wrapped)
 		   (cons (w/union union-name) defined))))
@@ -1710,7 +1698,7 @@
       ((udecl (decl-spec-list (stor-spec (extern)) ,type-spec)
 	      (init-declr (ptr-declr (pointer) (ident ,name))))
        ;; This needs to have a delay and handler
-       (let* ((udecl (expand-typerefs udecl udict (*defined*)))
+       (let* ((udecl (expand-typerefs udecl (*udict*) (*defined*)))
 	      (udecl (udecl-rem-type-qual udecl))
 	      (mdecl (udecl->mdecl udecl)))
 	 (cnvt-extern (car mdecl) (cdr mdecl)))
@@ -1719,7 +1707,7 @@
       ;; non-pointer
       ((udecl (decl-spec-list (stor-spec (extern)) ,type-spec)
 	      ,init-declr . ,rest)
-       (let* ((udecl (expand-typerefs udecl udict (*defined*)))
+       (let* ((udecl (expand-typerefs udecl (*udict*) (*defined*)))
 	      (udecl (udecl-rem-type-qual udecl))
 	      (mdecl (udecl->mdecl udecl)))
 	 (cnvt-extern (car mdecl) (cdr mdecl))
@@ -1965,9 +1953,9 @@
 ;; => (values wrapped defined)
 (define* (process-decls decls udict
 			#:optional (wrapped '()) (defined '())
-			#:key (declf (lambda (n) #t))
+			#:key (declf (lambda (k) #t))
 			)
-  (let* () ;;(declf (if declf declf (lambda (name) #t))))
+  (let* () ;;(declf (if declf declf (lambda (key) #t))))
     (fold-values			  ; from (sxml fold)
      (lambda (name wrapped defined) ; name: "foo_t" or (enum . "foo")
        (catch 'ffi-help-error
@@ -2076,7 +2064,8 @@
 	(lambda ()
 	  ;; We need to have externs in wrapped because function param
 	  ;; type have wrapped types preserved (e.g., enums).
-	  (process-decls ffi-decls udict
+	  ;; swap of udecls with udict failed on glib g???
+	  (process-decls ffi-decls udecls ;; udict <= swap failed 01 Dec 2018
 			 ;; wrapped and defined:
 			 ext-defd (append bs-defined ext-defd)
 			 ;; declaration filter
@@ -2115,13 +2104,14 @@
 ;; @end example
 ;; @end deffn
 (define (C-fun-decl->scm code)
-  (let* ((tree (with-input-from-string code parse-c99))
-	 (udict (unitize-decl (sx-ref tree 1)))
-	 (name (caar udict)) (udecl (cdar udict))
-	 (gen1 (fh-cnvt-udecl udecl '()))
-	 (gen2 (with-input-from-string gen1 read))
-	 (gen3 (caddr gen2)))
-    gen3))
+  (let ((tree (with-input-from-string code parse-c99)))
+    (if tree
+	(let* ((udict (unitize-decl (sx-ref tree 1)))
+	       (name (caar udict)) (udecl (cdar udict))
+	       (gen1 (fh-cnvt-udecl udecl '()))
+	       (gen2 (with-input-from-string gen1 read))
+	       (gen3 (caddr gen2)))
+	  gen3))))
 
 (define* (fh-cnvt-udecl udecl udict #:key (prefix "fh"))
   (parameterize ((*options* '()) (*wrapped* '()) (*defined* '())
@@ -2190,10 +2180,13 @@
 	   (udict (c99-trans-unit->udict/deep tree))
 	   (udecls (c99-trans-unit->udict tree))
 	   (decls (map car udecls))
-	   (scmfile ".fh_temp.scm") ; ugly hack, need tmpfile w/ .scm ending
+	   (scmport (mkstemp! (string-copy ",_FH_XXXXXX")))
+	   ;;(scmport (tmpfile)) ;; does not work ?
+	   (scmfile (port-filename scmport))
+	   (compile-file (@@ (system base compile) compile-file))
 	   )
       (*prefix* (symbol->string (gensym "fh-")))
-      (*mport* (open-output-file scmfile))
+      (*mport* scmport)
       (*udict* udict)
       (sfscm "(use-modules (system ffi-help-rt))\n")
       (sfscm "(use-modules ((system foreign) #:prefix ffi:))\n")
@@ -2204,8 +2197,8 @@
 			 (pkg-config-libs pkg-config)))))
       (process-decls decls udict '() bs-defined)
       (close (*mport*))
-      (load scmfile)
-      (delete-file scmfile)
+      (simple-format #t "wrote ~S; compile and load: ...\n" scmfile)
+      (load-compiled (compile-file scmfile #:opts '()))
       (if #f #f))))
 
 
