@@ -19,7 +19,7 @@
 
 (define-module (nyacc lang c99 pprint)
   #:export (pretty-print-c99)
-  #:use-module ((srfi srfi-1) #:select (pair-for-each))
+  #:use-module ((srfi srfi-1) #:select (pair-for-each fold-right))
   #:use-module (nyacc lang util)
   #:use-module (nyacc lang sx-util)
   #:use-module (ice-9 pretty-print)
@@ -181,15 +181,16 @@
 	(ppx/p rval)
 	(ppx rval)))
 
-  (define (pp-attr attr) ;; attributes
+  ;; now ((comment xxx) (attributes "aaa;yyy;zzz"))
+  (define (pp-attr attr)
     (string-join
-     (map
-      (lambda (val)
-	;;(simple-format #t "a: ~S\n" val)
-	(cond
-	 ((eq? (car val) 'packed) "__packed__")
-	 (else (symbol->string (car val)))))
-      attr)
+     (fold-right
+      (lambda (pair seed)
+	(if (eqv? 'attributes (car pair))
+	    ;; FIXME: should really parse-attributes, then ppx
+	    (append (string-split (cadr pair) #\;) seed)
+	    seed))
+      '() attr)
      " "))
 
   (define (struct-union-def struct-or-union attr name fields)
@@ -378,7 +379,7 @@
       ((comp-declr ,declr) (ppx declr))
       ((param-declr ,declr) (ppx declr))
       ;; This should work with sx-match, to replace above three.
-      ;;(((init-declr comp-declr param-declr) ,declr) (ppx declr))
+      ;;((#(init-declr comp-declr param-declr) ,declr) (ppx declr))
 
       ((bit-field ,ident ,expr)
        (ppx ident) (sf " : ") (ppx expr))
