@@ -24,13 +24,12 @@
 	    fh-type?
 	    fh-object? fh-object-val
 	    fh-object-ref fh-object-set!
-	    pointer-to value-at NULL
+	    pointer-to value-at NULL !0
 
 	    ;; maybe used outside of modules?
 	    fh:cast fh-cast bs-addr
 	    fh:function
 	    ffi-void*
-	    fh-void fh-void? make-fh-void
 	    make-fht
 
 	    ;; called from output of the ffi-compiler
@@ -56,9 +55,9 @@
   #:use-module (rnrs bytevectors)
   #:use-module ((system foreign) #:prefix ffi:)
   #:use-module (srfi srfi-9)
-  #:version (0 86 9))
+  #:version (0 90 0))
 
-(define *ffi-help-version* "0.86.9")
+(define *ffi-help-version* "0.90.0")
 
 (define (sferr fmt . args) (apply simple-format (current-error-port) fmt args))
 
@@ -153,6 +152,9 @@
     (throw 'ffi-help-error "expecting something I can dereference"))))
 
 (define NULL ffi:%null-pointer)
+(define (!0 v) (not (zero? v)))
+;;(define FALSE 0)
+;;(define TRUE 1)
 
 ;; === objects ============
 
@@ -301,7 +303,9 @@
 	      (fht-unwrap type)
 	      (fht-pointer-to type)
 	      (fht-value-at type)
-	      (fht-printer type))))
+	      ;;(fht-printer type)
+	      (make-printer (quote alias))
+	      )))
 
 ;; @deffn {Syntax} define-fh-compound-type type desc type? make
 ;; Generates an FY aggregate type based on a bytestructure descriptor.
@@ -604,13 +608,35 @@
 (define ffi-void* '*)
 
 (define char*-desc (bs:pointer 'void))
-(define-fh-pointer-type char* char*-desc char*? make-char*)
+(define char*
+  (make-fht 'char*
+	    (lambda (obj) (ffi:pointer->string (fh-object-ref obj)))
+	    (case-lambda
+	      ((val)
+	       (cond
+		((string? val)
+		 (bytestructure
+		  char*-desc (ffi:pointer-address (ffi:string->pointer val))))
+		((ffi:pointer? val)
+		 (bytestructure char*-desc (ffi:pointer-address val)))
+		(else
+		 (bytestructure char*-desc val))))
+	      (() (bytestructure char*-desc 0)))
+	    #f #f
+	    (make-bs-printer 'char*)))
+(define make-char* (fht-wrap char*))
+(define char*? (lambda (obj) (eq? (struct-vtable obj) char*)))
+(export char*-desc char* char*? make-char*)
+
 (define char**-desc (bs:pointer char*-desc))
 (define-fh-pointer-type char** char**-desc char**? make-char**)
+(export char**-desc char** char**? make-char**)
+
 (ref<->deref! char** make-char** char* make-char*)
+
 (define (char*->string obj)
   (ffi:pointer->string (ffi:make-pointer (fh-object-ref obj))))
-(export make-char* char*->string)
+(export char*->string)
 
 (define fh-void
   (make-fht 'void 
@@ -624,6 +650,54 @@
   (case-lambda
     (() (make-struct/no-tail fh-void 'void))
     ((val) (make-struct/no-tail fh-void 'void))))
+(export fh-void fh-void? make-fh-void)
+
+(define void*-desc (bs:pointer 'void))
+(define void*
+  (make-fht 'void*
+	    unwrap~pointer
+	    (case-lambda
+	      ((val)
+	       (cond
+		((string? val)
+		 (make-struct/no-tail
+		  void* (bytestructure
+			 void*-desc (ffi:pointer-address
+				     (ffi:string->pointer val)))))
+		((bytestructure? val)
+		 (make-struct/no-tail void* val))
+		(else
+		 (make-struct/no-tail void* (bytestructure void*-desc val)))))
+	      (() (make-struct/no-tail void* (bytestructure void*-desc))))
+	    #f #f
+	    (lambda (obj port)
+	      (display "#<void* 0x" port)
+	      (display (number->string (struct-ref obj 0) 16) port)
+	      (display ">" port))))
+(define make-void* (fht-wrap void*))
+(define void*? (lambda (obj) (eq? (struct-vtable obj) void*)))
+(ref<->deref! void* make-void* fh-void make-fh-void)
+(export void* void*? make-void*)
+
+(define void**-desc (bs:pointer (bs:pointer 'void)))
+(define void**
+  (make-fht 'void**
+	    unwrap~pointer
+	    (case-lambda
+	      ((val)
+	       (make-struct/no-tail void** (bytestructure void**-desc val)))
+	      (()
+	       (make-struct/no-tail void** (bytestructure void**-desc))))
+	    #f #f
+	    (lambda (obj port)
+	      (display "#<void** 0x" port)
+	      (display (number->string (struct-ref obj 0) 16) port)
+	      (display ">" port))))
+(define make-void** (fht-wrap void**))
+(define void**? (lambda (obj) (eq? (struct-vtable obj) void**)))
+(ref<->deref! void** make-void** void* make-void*)
+(export void** void**? make-void**)
+
 
 (define-syntax make-maker
   (syntax-rules ()
